-- ABR-Revived - by Kashi (kashi1988@gmail.com)
-- -- original Author: Wikwocket (wikwocket@gmail.com)
-- Displays an annoying reminder to use armor spells, auras, shouts, etc, and Victory Rush.

-- TODO
-- ui redesign
-- full localization

local VERSION = "r11";

local className = nil;
local inCombat = false;
local spec = GetSpecialization();
local level = UnitLevel("player");
local stance = GetShapeshiftFormID();
local currentTalents = {};

-- Print: helper function to send message to default chat frame
local function Print(text)
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(text);
	end
end

-- RefreshTalents: parses current talents
local function RefreshTalents()
	local talentRowCount = 6;
	
	for talentRow = 1,talentRowCount do
		rowIsFree, currentTalents[talentRow] = GetTalentRowSelectionInfo(talentRow);
		if (not rowIsFree) then
			currentTalents[talentRow] = currentTalents[talentRow] - (talentRow-1) * 3;
		else
			currentTalents[talentRow] = 0;
		end
	end
end
			
local function IDFromSlot(slot)
	local slotid = GetInventoryItemID("player", slot);
	return slotid;
end

-- OnLoad: register events, slash commands, and hooks, print welcome message
function ABRRevivedOnLoad()
	ABRRevivedHoverFrame:Hide();
	_, className = UnitClass("player");
	SlashCmdList["ABRRevived"] = ABRRevivedSlashCommand;
	SLASH_ABRRevived1 = "/ABRRevived";
	SLASH_ABRRevived2 = "/abrr";
	
	ABRRevivedHoverFrame:RegisterEvent("VARIABLES_LOADED");
	
	ABRRevivedHoverFrame:RegisterEvent("UNIT_AURA");
	ABRRevivedHoverFrame:RegisterEvent("UNIT_INVENTORY_CHANGED");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_LEVEL_UP");
	ABRRevivedHoverFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_TALENT_UPDATE");
	
	RefreshTalents();	
end

-- SlashCommand: handle slash commands
function ABRRevivedSlashCommand(command)
	if (command == "lock") then
		ABRRevivedVariables.lock = true;
		ABRRevivedHoverFrame:EnableMouse(false);
		Print("ABRRevived: Window locked. You can now click through the reminder window.");
	-- unlock frame
	elseif (command == "unlock") then
		ABRRevivedVariables.lock = false;
		ABRRevivedHoverFrame:EnableMouse(true);
		Print("ABRRevived: Window unlocked. You can now move the reminder window by clicking and dragging.");
	-- re-center frame
	elseif (command == "reset" or command == "center") then
		ABRRevivedHoverFrame:ClearAllPoints();
		ABRRevivedHoverFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
		Print("ABRRevived: Window position reset to center of screen.");
	-- toggle spell icon display
	elseif (command == "icon" or command == "icons") then
		ABRRevivedVariables.dontShowIcons = not ABRRevivedVariables.dontShowIcons;
		if (ABRRevivedVariables.dontShowIcons) then
			Print("ABRRevived: Spell icons will no longer be shown.");
		else
			Print("ABRRevived: Spell icons will now be shown with spell names.");
		end
	-- toggle stance/presence helpee
	elseif (command == "stancer") then
		ABRRevivedVariables.stancer = not ABRRevivedVariables.stancer;
		if (ABRRevivedVariables.stancer) then
			Print("ABRRevived: Stancer activated.");
		else
			Print("ABRRevived: Stancer deactivated.");
		end
	-- toggle procs helper
	elseif (command == "procs") then
		ABRRevivedVariablesPerChar.procs = not ABRRevivedVariablesPerChar.procs;
		if (ABRRevivedVariablesPerChar.procs) then
			Print("ABRRevived: Proc helper activated.");
		else
			Print("ABRRevived: Proc helper deactivated.");
		end
		
	-- -- debug / test commands
	-- display spec + level
	elseif (command == "test") then
		Print("ID of current spec: " .. spec);
		Print("Current Level: " .. level);
	-- stance
	elseif (command == "stance") then
		if (stance ~= nil) then
			Print("ID of current stance: " .. stance);
		else
			Print("No stance!");
		end
	-- buff list
	elseif (command == "buffs") then
		for i = 1, 40 do
			buffName = UnitAura("player", i, "HELPFUL");
			Print("buff " .. i .. " is " .. tostring(buffName));
		end
	-- talents
	elseif (command == "talents") then
		print("Talents:");
		for i = 1, 6 do
			if (currentTalents[i] == 1) then
				print("# - -");
			elseif (currentTalents[i] == 2) then
				print("- # -");
			elseif (currentTalents[i] == 3) then
				print("- - #");
			else
				print("- - -");
			end
		end
		
	-- default: print usage menu
	else
		Print("ABR-Revived " .. VERSION);
		Print("Displays an annoying reminder to use buffs, armor spells, etc.");
		Print("  /abrr icons - Toggle display of spell icons");
		Print("  /abrr lock||unlock - Toggle window locking and click-through");
		Print("  /abrr center - Reset window position to center of screen");
		Print("Additional Features:");
		Print("  /abrr stancer - Toggle stance/presence helper functionality");
		Print("  /abrr procs - Toggle proc helper functionality");
	end
end

-- OnEvent:
function ABRRevivedOnEvent(self, event, ...)
	local arg1 = select(1, ...);
	-- VARIABLES_LOADED: initialize
	if (event == "VARIABLES_LOADED") then
		-- load (or create) saved variables
		if (not ABRRevivedVariables) then
			ABRRevivedVariables = {};
			ABRRevivedVariables.lock = false;
			ABRRevivedVariables.stancer = true;
			-- let other vars init to nil/false
		end
		
		if (ABRRevivedVariables.stancer == nil) then 
			ABRRevivedVariables.stancer = true;
		end
		
		if (not ABRRevivedVariablesPerChar) then
			ABRRevivedVariablesPerChar = {};
			ABRRevivedVariablesPerChar.procs = false;
			ABRRevivedVariablesPerChar.procsWarn = true;
		end
		-- lock or unlock
		ABRRevivedHoverFrame:EnableMouse(not ABRRevivedVariables.lock);
		return;
	end
	
	-- PLAYER_REGEN_DISABLED: set combat flag
	if (event == "PLAYER_REGEN_DISABLED") then
		inCombat = true;
	end
	
	-- PLAYER_REGEN_ENABLED: clear combat flag
	if (event == "PLAYER_REGEN_ENABLED") then
		inCombat = false;
	end
	
	-- PLAYER_SPECIALIZATION_CHANGED: spec change -> update spec
	if (event == "PLAYER_SPECIALIZATION_CHANGED") then
		spec = GetSpecialization();
		RefreshTalents();
	end
	
	-- PLAYER_TALENT_UPDATE: talent change -> update talents
	if (event == "PLAYER_TALENT_UPDATE") then
		RefreshTalents();
	end
	
	-- PLAYER_LEVEL_UP: levelup -> update level
	if (event == "PLAYER_LEVEL_UP") then
		level = UnitLevel("player");
	end
	
	-- UPDATE_SHAPESHIFT_FORM: levelup -> update level
	if (event == "UPDATE_SHAPESHIFT_FORM") then
		stance = GetShapeshiftFormID();
	end
	
	-- abort if inappropriate to notify player now
	if (IsFlying() or UnitOnTaxi("player") or IsResting() or not HasFullControl()) then
			ABRRevivedHoverFrame:Hide();
			return;
	end
	
	-- vars for whether ABRR should be shown, used in code blocks below
	local showABRR = false;
	local textABRR = "";
	local spellABRR = ""; -- used for determining spell icon texture
	local spellABRRalt = ""; -- used for determining 2nd spell icon texture
	local itemABRR = "";
	local buffName;
	local casterID; -- note: only used for classes for whom all of the buffs we check are self-cast only
	
	if (not(event == "UNIT_AURA" and arg1 ~= "player")) then -- UNIT_AURA only if player triggered
		if (className == "PRIEST") then
			-- inner fire, pw:fort
			local needI = true;
			local needPWF = true;
			local needSF = true;
			local needChakra = true;
			if (spec == 2 and level >= 56) then
				chakraCooldownStart, chakraCooldownDuration  = GetSpellCooldown(81206);
				if (chakraCooldownStart > 0 and chakraCooldownDuration > 0) then
					needChakra = false;
				end
			end
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				-- Print("buff " .. i .. " is " .. tostring(buffName) .. "/" .. tostring(casterID));
				if (buffName) then
					if (buffName == ABRR_LOCALE.innerFire
						or buffName == ABRR_LOCALE.innerWill) then
						needI = false;
					elseif (buffName == ABRR_LOCALE.powerFortitude
						or buffName == ABRR_LOCALE.darkIntent
						or buffName == ABRR_LOCALE.songOfVictory) then
							needPWF = false;
					elseif (buffName == ABRR_LOCALE.shadowform) then
							needSF = false;
					elseif (buffName == ABRR_LOCALE.chakraAoE
						or buffName == ABRR_LOCALE.chakraSingle
						or buffName == ABRR_LOCALE.chakraDPS) then
							needChakra = false;
					end
				end
			end
			if (needI and level >= 9) then
				if (level >= 80) then
					showABRR = true;
					textABRR = "Cast Inner Fire/Will!";
					spellABRR = ABRR_LOCALE.innerFire;
					spellABRRalt = ABRR_LOCALE.innerWill;
				else
					showABRR = true;
					textABRR = "Cast Inner Fire!";
					spellABRR = ABRR_LOCALE.innerFire;
				end
			elseif (needPWF and level >= 22) then
				showABRR = true;
				textABRR = "Cast PW:Fortitude!";
				spellABRR = ABRR_LOCALE.powerFortitude;
			elseif (needSF and spec == 3 and level >= 24) then
				showABRR = true;
				textABRR = "Shadowform!";
				spellABRR = ABRR_LOCALE.shadowform;
			elseif (needChakra and spec == 2 and level >= 56) then
				showABRR = true;
				textABRR = "Chakra!";
				spellABRR = ABRR_LOCALE.chakraSingle;
				spellABRRalt = ABRR_LOCALE.chakraAoE;
			end
		elseif (className == "WARLOCK") then
			--  dark intent,sacrifice
			local needDI = true;
			local needSacrifice = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.powerFortitude and
							(buffName == ABRR_LOCALE.arcaneBrilliance
						or buffName == ABRR_LOCALE.dalaranBrilliance)
						or buffName == ABRR_LOCALE.darkIntent) then
							needDI = false;
					elseif (buffName == ABRR_LOCALE.sacrifice) then
							needSacrifice = false;
					end
				end
			end
			if (needDI and level >= 82) then
				showABRR = true;
				textABRR = "Cast an Dark Intent!";
				spellABRR = ABRR_LOCALE.darkIntent;
			elseif (needSacrifice and currentTalents[5] == 3) then
				showABRR = true;
				textABRR = "Sacrifice your Demon!";
				spellABRR = ABRR_LOCALE.sacrifice;
			end
		elseif (className == "MAGE") then
			-- armor spell, ai
			local needArmor = true;
			local needAI = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.frostArmor
						or buffName == ABRR_LOCALE.mageArmor
						or buffName == ABRR_LOCALE.moltenArmor) then
							needArmor = false;
					elseif (buffName == ABRR_LOCALE.arcaneBrilliance
						or buffName == ABRR_LOCALE.dalaranBrilliance) then
							needAI = false;
					end
				end
			end
			if (needArmor and level >= 34) then
				if (level >= 54) then
					showABRR = true;
					textABRR = "Cast an Armor!";
					spellABRR = ABRR_LOCALE.moltenArmor;
					spellABRRalt = ABRR_LOCALE.frostArmor;
				else
					showABRR = true;
					textABRR = "Cast an Molten Armor!";
					spellABRR = ABRR_LOCALE.moltenArmor;
				end
			elseif (needAI and level >= 58) then
				showABRR = true;
				textABRR = "Cast Arcane Int!";
				spellABRR = ABRR_LOCALE.arcaneBrilliance;
			end
		elseif (className == "SHAMAN") then
			-- main hand weapon enchant
			local mainHandEnchanted, _, _, offHandEnchanted = GetWeaponEnchantInfo();
			if (not mainHandEnchanted and level >= 10) then
				showABRR = true;
				textABRR = "Enchant Main Hand!";
				if (spec == 2) then
					spellABRR = ABRR_LOCALE.windfuryWeapon;
				elseif (spec == 3) then
					spellABRR = ABRR_LOCALE.earthlivingWeapon;
				else
					spellABRR = ABRR_LOCALE.flametongueWeapon;
				end
			end
			-- off hand weapon enchant
			if (not showABRR and level >= 10) then
				local offHandLink = GetInventoryItemLink("player", 17);
				if (offHandLink) then
					local _, _, _, _, _, itemType = GetItemInfo(offHandLink);
					if (itemType == ABRR_LOCALE.itemTypeWeapon and not offHandEnchanted) then
						showABRR = true;
						textABRR = "Enchant Off Hand!";
						spellABRR = ABRR_LOCALE.flametongueWeapon;
					end
				end
			end
			-- shield spell
			if (not showABRR and level >= 8) then
				local needShield = true;
				for i = 1, 40 do
					-- use PLAYER filter and casterID since only need to track *player*-cast spells
					buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
					if (buffName and casterID == "player") then
						if ((buffName == ABRR_LOCALE.lightningShield and (spec == 1 or spec == 2))
							or ((buffName == ABRR_LOCALE.waterShield
							or buffName == ABRR_LOCALE.earthShield) and spec == 3)) then
								needShield = false;
						end
					end
				end
				if (needShield) then
					showABRR = true;
					textABRR = "Cast a Shield!";
					if ((spec == 1 or spec == 2) and level >= 8) then
						spellABRR = ABRR_LOCALE.lightningShield;
					elseif (spec == 3 and level >= 20) then
						spellABRR = ABRR_LOCALE.waterShield;
					elseif (level >= 8) then
						spellABRR = ABRR_LOCALE.lightningShield;
					end
				end
			end
		elseif (className == "HUNTER") then
			-- aspect
			local needAspect = true;
			for i = 1, 40 do
				-- use PLAYER filter and casterID since only need to track *player*-cast spells
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
				if (buffName and casterID == "player") then
					if (string.match(buffName, ABRR_LOCALE.regexAspect)) then
						needAspect = false;
					end
					if (inCombat and (buffName == ABRR_LOCALE.aspectPack or buffName == ABRR_LOCALE.aspectCheetah)) then
						needAspect = true;
					end
				end
			end
			if (needAspect and level >= 12) then
				showABRR = true;
				textABRR = "Cast an Aspect!";
				spellABRR = ABRR_LOCALE.aspectHawk;
			end
		elseif (className == "ROGUE") then
			-- poisons
			local needDPSPoison = true;
			local needSlowPoison = true;
			for i = 1, 40 do
				-- use PLAYER filter and casterID since only need to track *player*-cast spells
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
				if (buffName and casterID == "player") then
					if (buffName == ABRR_LOCALE.mindPoison or buffName == ABRR_LOCALE.slowPoison
							or buffName == ABRR_LOCALE.egelPoison or buffName == ABRR_LOCALE.betPoison) then
						needSlowPoison = false;
					elseif (buffName == ABRR_LOCALE.msPoison or buffName == ABRR_LOCALE.deadlyPoison) then
						needDPSPoison = false;
					end
				end
			end
			if (needSlowPoison and level >= 28) then
				showABRR = true;
				textABRR = "Apply Slow-Poison!";
				spellABRRalt = ABRR_LOCALE.slowPoison;
				spellABRR = ABRR_LOCALE.mindPoison;
			elseif (needSlowPoison and level >= 20) then
				showABRR = true;
				textABRR = "Apply Slow-Poison!";
				spellABRRalt = ABRR_LOCALE.slowPoison;
			elseif (needDPSPoison and level >= 30) then
				showABRR = true;
				textABRR = "Apply DPS-Poison!";
				spellABRR = ABRR_LOCALE.deadlyPoison;
				spellABRRalt = ABRR_LOCALE.msPoison;
			elseif (needDPSPoison and level >= 10) then
				showABRR = true;
				textABRR = "Apply Deadly Poison!";
				spellABRR = ABRR_LOCALE.deadlyPoison;
			end
		elseif (className == "PALADIN") then
			-- blessingKings, blessingMight
			local needKings = true;
			local needMight = true;
			for i = 1, 40 do
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.markWild
						or buffName == ABRR_LOCALE.blessingKings
						or buffName == ABRR_LOCALE.legacyEmperor
						or (buffName == ABRR_LOCALE.blessingMight and casterID == "player")) then
						needKings = false;
					end
					if ((buffName == ABRR_LOCALE.blessingKings and casterID == "player")
						or buffName == ABRR_LOCALE.blessingMight) then
						needMight = false;
					-- elseif (string.match(buffName, ABRR_LOCALE.regexSeal)) then
					--	needSeal = false;
					end
				end
			end
			if (needKings and level >= 30) then
				showABRR = true;
				textABRR = "Cast a Blessing of Kings!";
				spellABRR = ABRR_LOCALE.blessingKings;
			elseif (needMight and level >= 81) then
				showABRR = true;
				textABRR = "Cast a Blessing of Might!";
				spellABRR = ABRR_LOCALE.blessingMight;
			end
		elseif (className == "DRUID") then
			-- motw, symbiosis
			local needMark = true;
			local needSymbiosis = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.markWild
						or buffName == ABRR_LOCALE.blessingKings
						or buffName == ABRR_LOCALE.legacyEmperor) then
						needMark = false;
					elseif (buffName == ABRR_LOCALE.symbiosis) then
						needSymbiosis = false;
					end
				end
			end
			if (needMark and level >= 62) then
				showABRR = true;
				textABRR = "Cast Mark of the Wild!";
				spellABRR = ABRR_LOCALE.markWild;
			elseif (needSymbiosis and level >= 87
						and not inCombat and GetNumGroupMembers() > 5) then
				showABRR = true;
				textABRR = "Cast Symbiosis!";
				spellABRR = ABRR_LOCALE.symbiosis;
			end
		elseif (className == "DEATHKNIGHT") then
			-- presence, horn of winter
			local presence = "none";
			local needHorn = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.bloodPresence) then
						presence = "blood";
					elseif (buffName == ABRR_LOCALE.frostPresence) then
						presence = "frost";
					elseif (buffName == ABRR_LOCALE.unholyPresence) then
						presence = "unholy";
					elseif (buffName == ABRR_LOCALE.hornOfWinter) then
						needHorn = false;
					end
				end
			end
			if (level >=64 and ABRRevivedVariables.stancer) then
				if (presence ~= "none" and level >= 64) then
					if ((spec == 2 or spec == 3) and presence == "blood") then
						showABRR = true;
						spellABRR = ABRR_LOCALE.frostPresence;
						spellABRRalt = ABRR_LOCALE.unholyPresence;
						textABRR = "Switch Presence!";
					elseif (spec == 1 and (presence == "frost" or presence == "unholy")) then
						showABRR = true;
						spellABRR = ABRR_LOCALE.bloodPresence;
						textABRR = "Switch Presence!";
					end
				else
					showABRR = true;
					textABRR = "Activate Presence!";
					if (spec == 2 or spec == 3) then
						spellABRR = ABRR_LOCALE.frostPresence;
						spellABRRalt = ABRR_LOCALE.unholyPresence;
					elseif (spec == 1) then
						spellABRR = ABRR_LOCALE.bloodPresence;
					end
				end
			end
			if (inCombat and level >=65 and needHorn == true) then
						showABRR = true;
						spellABRR = ABRR_LOCALE.hornOfWinter;
						textABRR = "Horn of Winter!";
			end
		elseif (className == "MONK") then
			-- legacyEmperor, legacyWhiteTiger
			local needEmperor = true;
			local needWhiteTiger = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.markWild
						or buffName == ABRR_LOCALE.blessingKings
						or buffName == ABRR_LOCALE.legacyEmperor) then
						needEmperor = false;
					elseif (buffName == ABRR_LOCALE.legacyWhiteTiger) then
						needWhiteTiger = false;
					end
				end
			end
			if (level >= 10 and ABRRevivedVariables.stancer) then
				if (spec == 1  and stance ~= 23) then
					showABRR = true;
					textABRR = "Switch Stance!";
					spellABRR = ABRR_LOCALE.tankStance;
				elseif (spec == 2  and stance ~= 20) then
					showABRR = true;
					textABRR = "Switch Stance!";
					spellABRR = ABRR_LOCALE.healStance;
				elseif (spec == 3  and stance ~= 24) then
					showABRR = true;
					textABRR = "Switch Stance!";
					spellABRR = ABRR_LOCALE.dpsStance;
				end
			end
			if (needEmperor and level >= 22) then
				showABRR = true;
				textABRR = "Cast Legacy of the Emperor!";
				spellABRR = ABRR_LOCALE.legacyEmperor;
			elseif (needWhiteTiger and spec == 3 and level >= 81) then
				showABRR = true;
				textABRR = "Cast Legacy of the White Tiger!";
				spellABRR = ABRR_LOCALE.legacyWhiteTiger;
			end
		elseif (className == "WARRIOR") then
			if (level >=9 and ABRRevivedVariables.stancer) then
				if ((spec == 1 or spec == 2) and stance == 18) then
					showABRR = true;
					textABRR = "Switch Stance!";
					spellABRR = ABRR_LOCALE.stanceBattle17;
					if (level >= 34) then
						spellABRRalt = ABRR_LOCALE.stanceBerserk19;
					end
				elseif (spec == 3 and (stance == 17 or stance == 19)) then
					showABRR = true;
					textABRR = "Switch Stance!";
					spellABRR = ABRR_LOCALE.stanceDef18;
				end
			end
		end -- end classes ifelseif
		
		-- wrong item reminder code
-- cloak
		if (IDFromSlot(15) == 65360					-- teleport cloak	alliance	epic
		or IDFromSlot(15) == 63206					-- 											rare
		or IDFromSlot(15) == 63352					-- 											uncommon
		or IDFromSlot(15) == 65274					--							horde		epic
		or IDFromSlot(15) == 63207					--											rare
		or IDFromSlot(15) == 63353					--											uncommon
		) then
			showABRR = true;
			textABRR = "Switch Cloak!";
			itemABRR = IDFromSlot(15);
-- head
		elseif (IDFromSlot(1) == 46349				-- chefs hat
		or IDFromSlot(1) == 65665						-- pirate gnome transform hood
		) then
			showABRR = true;
			textABRR = "Switch Head!";
			itemABRR = IDFromSlot(1);
-- hands
		elseif (IDFromSlot(10) == 46349				-- herbing gloves
		) then
			showABRR = true;
			textABRR = "Switch Gloves!";
			itemABRR = IDFromSlot(10);
-- main hand
		elseif (IDFromSlot(16) == 44050			-- fishing rods start
		or IDFromSlot(16) == 25978
		or IDFromSlot(16) == 19022
		or IDFromSlot(16) == 6367
		or IDFromSlot(16) == 6366
		or IDFromSlot(16) == 84661
		or IDFromSlot(16) == 19970
		or IDFromSlot(16) == 45858
		or IDFromSlot(16) == 45992
		or IDFromSlot(16) == 45991
		or IDFromSlot(16) == 6365
		or IDFromSlot(16) == 12225
		or IDFromSlot(16) == 46337
		or IDFromSlot(16) == 84660
		or IDFromSlot(16) == 6256						-- fishing rods end
		) then
			showABRR = true;
			textABRR = "Switch Main Hand!";
			itemABRR = IDFromSlot(16);
-- upper ring
		elseif (IDFromSlot(11) == 51557			-- dalaran rings start
		or IDFromSlot(11) == 51558
		or IDFromSlot(11) == 51559
		or IDFromSlot(11) == 51560
		or IDFromSlot(11) == 48954
		or IDFromSlot(11) == 48955
		or IDFromSlot(11) == 48956
		or IDFromSlot(11) == 48957
		or IDFromSlot(11) == 45688
		or IDFromSlot(11) == 45689
		or IDFromSlot(11) == 45690
		or IDFromSlot(11) == 45691
		or IDFromSlot(11) == 40585
		or IDFromSlot(11) == 40586
		or IDFromSlot(11) == 44934
		or IDFromSlot(11) == 44935					-- dalaran rings end
		or IDFromSlot(11) == 95051					-- brawler ring	alliance
		or IDFromSlot(11) == 95050					--						horde
		) then
			showABRR = true;
			textABRR = "Switch Upper Ring!";
			itemABRR = IDFromSlot(11);
-- lower ring
		elseif (IDFromSlot(12) == 51557			-- dalaran rings start
		or IDFromSlot(12) == 51558
		or IDFromSlot(12) == 51559
		or IDFromSlot(12) == 51560
		or IDFromSlot(12) == 48954
		or IDFromSlot(12) == 48955
		or IDFromSlot(12) == 48956
		or IDFromSlot(12) == 48957
		or IDFromSlot(12) == 45688
		or IDFromSlot(12) == 45689
		or IDFromSlot(12) == 45690
		or IDFromSlot(12) == 45691
		or IDFromSlot(12) == 40585
		or IDFromSlot(12) == 40586
		or IDFromSlot(12) == 44934
		or IDFromSlot(12) == 44935					-- dalaran rings end
		or IDFromSlot(12) == 95051					-- brawler ring	alliance
		or IDFromSlot(12) == 95050					--						horde
		) then					--
			showABRR = true;
			textABRR = "Switch Lower Ring!";
			itemABRR = IDFromSlot(12);
		end -- wrong item reminder code end

		ABRRevivedHoverFrame:SetBackdrop({
			bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
    		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeSize = 2,
    		insets = {top = 1, left = 1, bottom = 1, right = 1}, tile = true
            });
    	ABRRevivedHoverFrame:SetBackdropColor(0 , 0 , 0 , 1);
    	ABRRevivedHoverFrame:SetBackdropBorderColor(0 , 0 , 0 , 1);

    	ABRRevivedSpellButton1:SetBackdrop({
			bgFile=nil,
    		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeSize = 2,
    		insets = {top = 1, left = 1, bottom = 1, right = 1}, tile = true
            });
    	ABRRevivedSpellButton1:SetBackdropColor(0 , 0 , 0 , 0);
    	ABRRevivedSpellButton1:SetBackdropBorderColor(0 , 0 , 0 , 1);

    	ABRRevivedSpellButton2:SetBackdrop({
			bgFile=nil,
    		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeSize = 2,
    		insets = {top = 1, left = 1, bottom = 1, right = 1}, tile = true
            });
    	ABRRevivedSpellButton2:SetBackdropColor(0 , 0 , 0 , 0);
    	ABRRevivedSpellButton2:SetBackdropBorderColor(0 , 0 , 0 , 1);
		
		-- display or hide ABRR as needed
		if (showABRR) then
			ABRRevivedText:SetText(textABRR);
			ABRRevivedHoverFrame:Show();
			if (ABRRevivedVariables.dontShowIcons) then
				ABRRevivedSpellButton1Icon:Hide();
				ABRRevivedSpellButton2Icon:Hide();
				ABRRevivedHoverFrame:SetWidth(200);
			else
				local texture = select(3, GetSpellInfo(spellABRR));
				if (spellABRR == "") then
					texture = GetItemIcon(itemABRR);
					ABRRevivedSpellButton2Icon:SetTexture(texture);
				elseif (spellABRRalt == "") then
					ABRRevivedSpellButton2Icon:SetTexture(texture);
				else
					local texturealt = select(3, GetSpellInfo(spellABRRalt));
					ABRRevivedSpellButton2Icon:SetTexture(texturealt);
				end	
				ABRRevivedSpellButton1Icon:SetTexture(texture);
				ABRRevivedSpellButton1Icon:Show();
				ABRRevivedSpellButton2Icon:Show();
				ABRRevivedHoverFrame:SetWidth(265);
			end
		else
			ABRRevivedHoverFrame:Hide();
		end
		return;
	end -- end UNIT_MANA/UNIT_AURA if
end
